package com.example.venue_reservation_service.service.impl;

import cn.hutool.jwt.JWTUtil;
import com.auth0.jwt.JWT;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.venue_reservation_service.domain.*;
import com.example.venue_reservation_service.dto.*;
import com.example.venue_reservation_service.mapper.AdminMapper;
import com.example.venue_reservation_service.service.*;
import com.example.venue_reservation_service.mapper.UserMapper;
import com.example.venue_reservation_service.utils.*;
import com.example.venue_reservation_service.vo.LoginVo;
import com.example.venue_reservation_service.vo.QueryVO;
import com.example.venue_reservation_service.vo.Result;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import jakarta.annotation.Resource;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

/**
* @author 31586
* @description 针对表【venue_user】的数据库操作Service实现
* @createDate 2024-11-12 21:43:55
*/
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
    implements UserService{
    @Resource
    private BalanceService balanceService;

    @Resource
    private PaymentService paymentService;

    @Resource
    private RestTemplate restTemplate;

    @Value("${weixin.app_id}")
    private String APP_ID;

    @Value("${weixin.secret}")
    private String SECRET;

    @Resource
    private SchoolService schoolService;

    @Resource
    private AdminMapper adminMapper;

    @Resource
    private RedisUtil redisUtil;

    @Value("${spring.application.name}")
    private String name;


    @Override
    public Result getOpenId(String code) {
        Map<String, String> vo = restTemplate.getForObject("https://api.weixin.qq.com/sns/jscode2session?"+
                        "appid=" + APP_ID +//小程序ID
                        "&secret=" + SECRET + //小程序密钥
                        "&js_code=" + code + //前端 传入 code
                        "&grant_type=authorization_code", Map.class);
        // 获取openid
        String openid = vo.get("openid");
        System.out.println(openid);
        if(openid == null || "".equals(openid)){
            return Result.fail().message("登录失败");
        }
        User user = getOne(Wrappers.<User>lambdaQuery().eq(User::getOpenId, openid));
        if(Optional.ofNullable(user).isEmpty()){
            // 如果当前用户信息不存在，进行注册
            user = new User();
            user.setUsername("用户" + System.currentTimeMillis());
            user.setOpenId(openid);
            user.setType(0);
            user.setAccount(LocalDate.now().getYear() +"-"+ System.currentTimeMillis());
            user.setRegisterTime(LocalDateTime.now());
            save(user);

            // 添加新的账户余额信息
            Balance balance = new Balance(0.0, user.getId());
            balanceService.save(balance);
        }
        // 获取用户账号余额数据
        Double money = balanceService.getOne(Wrappers.<Balance>lambdaQuery().eq(Balance::getUserId, user.getId()))
                .getBalance();
        user.setBalance(money);
        LoginVo loginVo = new LoginVo(user, JwtUtil.createToken(user.getId()));
        return Result.ok(loginVo).message("登录成功");
    }

    @Override
    public Result adminLogin(AdminDTO dto) {
        String account = dto.getAccount();
        LambdaQueryWrapper<User> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(User::getUsername, account);
        User user = getOne(wrapper);
        if (getOne(wrapper) == null) {
            return Result.fail().message("管理员账号不存在");
        }
        String password = dto.getPasswd();
        if(!user.getAccessCode().equals(password)){
            // 查询该账号在五分钟内的登录次数
            String key = name + dto.getAccount();
            if (Optional.ofNullable(redisUtil.get(key)).isEmpty()) {
                // 若该 key 值未出现过（或者过期了），则存入缓存，同时设置存活时间为5分钟
                redisUtil.set(key, 1+"-"+ System.currentTimeMillis(), 5 * 60);
            }else {
                String[] split = String.valueOf(redisUtil.get(key)).split("-");
                // 获取次数
                int i = Integer.parseInt(split[0]);
                if(i == 3){
                    return Result.fail().message("该账号已被锁定，请在 "+ DateUtil.calc(split[1])+" 秒后，再进行登录");
                }else{
                    i++;
                    redisUtil.set(key, i+"-"+String.valueOf(System.currentTimeMillis()), 5 * 60 );
                }
            }
            return Result.fail().message("账号密码不匹配");
        }
        user.setAccessCode("");

        LoginVo vo = new LoginVo(user, JwtUtil.createToken(user.getId()));
        return Result.ok(vo).message("登录成功");
    }

    @Override
    public Result userUpdate(User user) {
        String username = getById(user.getId()).getUsername();
        if(!username.equals(user.getUsername())){
            // 表示用户修改了用户名
            User one = getOne(Wrappers.<User>lambdaQuery()
                    .eq(User::getUsername, user.getUsername()));
            if (Optional.ofNullable(one).isPresent()) {
                return Result.fail().message("当前用户名已经被使用");
            }
        }
        updateById(user);
        return Result.ok(user).message("用户信息修改成功");
    }

    @Override
    public Result getUserList(UserDTO dto) {
        Integer userId = dto.getUserId();
        User admin = getById(userId);
        if(Optional.ofNullable(admin).isEmpty() || admin.getType() !=4) {
            return Result.fail().message("非超级管理员");
        }
        Page<User> page = new Page(dto.getCurrent(), dto.getSize());
        MPJLambdaWrapper<User> wrapper = new MPJLambdaWrapper<>();
        wrapper.selectAll(User.class)
                .select(Balance::getBalance)
                .leftJoin(Balance.class, Balance::getUserId, User::getId)
                .between(User::getType, 0, 2);
        page = page(page, wrapper);
        List<User> records = page.getRecords();
        records.forEach(item -> {
            // 对用户信息进行脱敏处理
            String phoneNumber = item.getUserPhone();
            if(Optional.ofNullable(phoneNumber).isEmpty() || phoneNumber.length() == 0){
                item.setUserPhone( "暂无用户联系方式");
            }else {
                if (phoneNumber.length() == 11){
                    item.setUserPhone(phoneNumber.substring(0, 3) + "****" + phoneNumber.substring(7));
                }
            }

            item.setEmail(PhoneUtil.maskEmail(item.getEmail()));
        });
        QueryVO<User> vo = new QueryVO<>();
        vo.setTotal(page.getTotal());
        vo.setList(records);
        return Result.ok(vo).message("ok");
    }

    @Override
    public Result userCheck(CheckDTO dto) {
        return Result.ok( ).message("身份信息认证成功");
    }

    @Override
    public Result getSchool(Integer userId) {
        School one = schoolService.getOne(Wrappers.<School>lambdaQuery()
                .eq(School::getUserId, userId));
        return Result.ok(one).message("ok");
    }

    @Override
    public Result userRecharge(Integer userId, Double amount) {
        Balance balance = balanceService.getOne(Wrappers.<Balance>lambdaQuery()
                .eq(Balance::getUserId, userId));
        if (Optional.ofNullable(balance).isEmpty()) {
            return Result.fail().message("未查询到账户信息");
        }
        Double aDouble = balance.getBalance();
        balance.setBalance(aDouble + amount);
        balanceService.updateById(balance);
        // 记录此次充值记录
        Payment payment = new Payment();
        payment.setUserId(userId);
        payment.setAmount(amount);
        payment.setSituation("账户余额充值");
        payment.setTime(LocalDateTime.now());
        paymentService.save(payment);
        return Result.ok().message("充值成功");
    }

    @Override
    public Result checkPhone(PhoneDTO dto) {
        User user = getById(dto.getUserId());
        if (Optional.ofNullable(user).isEmpty()) {
            return Result.fail().message("用户信息不存在");
        }

        // 校验手机号码是否正确
        if (!PhoneUtil.isValidPhoneNumber(dto.getPhone())) {
            return Result.fail().message("手机号格式错误");
        }
        // 简单校验
        if (dto.getPhone() == null){
            return Result.fail().message("手机号格式错误");
        }

        user.setUserPhone(dto.getPhone());
        user.setEmail(dto.getEmail());
        updateById(user);
        // 获取用户余额信息
        Double balance = balanceService.getOne(Wrappers.<Balance>lambdaQuery().eq(Balance::getUserId, user.getId())).getBalance();
        user.setBalance(balance);
        return Result.ok(user).message("手机号信息设置成功");
    }

    @Override
    public Result useNum() {
        Long count = getBaseMapper().selectCount(Wrappers.<User>lambdaQuery().in(User::getType, 0, 1, 2));
        return Result.ok(count).message("ok");
    }

    @Override
    public Result createAdmin(User user) {
        String phone = user.getUserPhone();
        if (!StringUtils.isNotBlank(phone)) {
            return Result.fail().message("管理员联系电话不能为空");
        }
        User one = getOne(Wrappers.<User>lambdaQuery().eq(User::getUserPhone, phone));
        if (Optional.ofNullable(one).isPresent()) {
            return Result.fail().message("该电话号码已经被使用");
        }
        // 生成随机密码
        user.setAccessCode(MD5Util.generateRandomString(16));
        user.setType(3); // 管理员
        user.setOpenId("ecjtu-admin");
        save(user);
        return Result.ok().message("管理员信息添加成功");
    }

    @Override
    public Result removeAdmin(Integer adminId) {
        Admin msg = adminMapper.selectOne(Wrappers.<Admin>lambdaQuery().eq(Admin::getUserId, adminId));
        if (Optional.ofNullable(msg).isPresent()) {
            return Result.fail().message("当前管理员已在管理场馆，请先进行取消");
        }
        removeById(adminId);
        return Result.ok().message("移除成功");
    }


    @Override
    public Result admininfo(PageDTO dto) {
        Page<User> page = new Page<>(dto.getCurrent(), dto.getSize());
        page = page(page, Wrappers.<User>lambdaQuery().eq(User::getType, 3));
        List<User> users = page.getRecords();
        users.forEach(item -> {
            // 对用户信息进行脱敏处理
            String phoneNumber = item.getUserPhone();
            if(Optional.ofNullable(phoneNumber).isEmpty() || phoneNumber.length() == 0){
                item.setUserPhone( "暂无用户联系方式");
            }else {
                if (phoneNumber.length() == 11){
                    item.setUserPhone(phoneNumber.substring(0, 3) + "****" + phoneNumber.substring(7));
                }
            }
            item.setEmail(PhoneUtil.maskEmail(item.getEmail()));
        });
        QueryVO<User> vo = new QueryVO<>();
        vo.setList(users);
        vo.setTotal(page.getTotal());
        return Result.ok(vo).message("查询成功");
    }

    @Override
    public Result passwd(PasswdDTO dto) {
        Integer userId = dto.getId();
        User user = getById(userId);
        String passwd = user.getAccessCode();
        if(!passwd.equals(dto.getOldPasswd())){
            return Result.fail().message("原密码输入错误");
        }
        String newPasswd = dto.getNewPasswd();
        if (passwd.equals(newPasswd)) {
            return Result.fail().message("新旧密码不能一致");
        }
        if (!newPasswd.equals(dto.getRePasswd())) {
            return Result.fail().message("两次输入密码不一致");
        }
        if (!MD5Util.validateString(newPasswd)) {
            return Result.fail().message("新密码至少长度大于10，同时至少包含大小写字符、数字和特殊字符");
        }
        user.setAccessCode(newPasswd);
        updateById(user);
        return Result.ok().message("密码修改成功");
    }
}




